@@ -432,7 +432,7 @@ class IgnoreIPDNS(LogCaptureTestCase):
 
 	def testIgnoreIPDNSOK(self):
 		self.filter.addIgnoreIP("www.epfl.ch")
-		self.assertTrue(self.filter.inIgnoreIPList("128.178.50.12"))
+		self.assertTrue(self.filter.inIgnoreIPList("128.178.222.69"))
 		self.filter.addIgnoreIP("example.com")
 		self.assertTrue(self.filter.inIgnoreIPList("93.184.216.34"))
 		self.assertTrue(self.filter.inIgnoreIPList("2606:2800:220:1:248:1893:25c8:1946"))
@@ -440,9 +440,9 @@ class IgnoreIPDNS(LogCaptureTestCase):
 	def testIgnoreIPDNSNOK(self):
 		# Test DNS
 		self.filter.addIgnoreIP("www.epfl.ch")
-		self.assertFalse(self.filter.inIgnoreIPList("127.177.50.10"))
-		self.assertFalse(self.filter.inIgnoreIPList("128.178.50.11"))
-		self.assertFalse(self.filter.inIgnoreIPList("128.178.50.13"))
+		self.assertFalse(self.filter.inIgnoreIPList("127.178.222.69"))
+		self.assertFalse(self.filter.inIgnoreIPList("128.178.222.68"))
+		self.assertFalse(self.filter.inIgnoreIPList("128.178.222.70"))
 
 	def testIgnoreCmdApacheFakegooglebot(self):
 		if not STOCK: # pragma: no cover
@@ -450,7 +450,7 @@ class IgnoreIPDNS(LogCaptureTestCase):
 		cmd = os.path.join(STOCK_CONF_DIR, "filter.d/ignorecommands/apache-fakegooglebot")
 		## below test direct as python module:
 		mod = Utils.load_python_module(cmd)
-		self.assertFalse(mod.is_googlebot(mod.process_args([cmd, "128.178.50.12"])))
+		self.assertFalse(mod.is_googlebot(mod.process_args([cmd, "128.178.222.69"])))
 		self.assertFalse(mod.is_googlebot(mod.process_args([cmd, "192.0.2.1"])))
 		bot_ips = ['66.249.66.1']
 		for ip in bot_ips:
@@ -980,6 +980,21 @@ def get_monitor_failures_testcase(Filter_):
 			self.assert_correct_last_attempt(GetFailures.FAILURES_01)
 			self.assertEqual(self.filter.failManager.getFailTotal(), 6)
 
+		def test_pyinotify_delWatch(self):
+			if hasattr(self.filter, '_delWatch'): # pyinotify only
+				m = self.filter._FilterPyinotify__monitor
+				# remove existing watch:
+				self.assertTrue(self.filter._delWatch(m.get_wd(self.name)))
+				# mockup get_path to allow once find path for invalid wd-value:
+				_org_get_path = m.get_path
+				def _get_path(wd):
+					#m.get_path = _org_get_path
+					return 'test'
+				m.get_path = _get_path
+				# try remove watch using definitely not existing handle:
+				self.assertFalse(self.filter._delWatch(0x7fffffff))
+				m.get_path = _org_get_path
+
 		def test_del_file(self):
 			# test filter reaction by delete watching file:
 			self.file.close()
@@ -997,6 +1012,7 @@ def get_monitor_failures_testcase(Filter_):
 			self.file.close()
 			self.filter.setMaxRetry(10)
 			self.filter.delLogPath(self.name)
+			_killfile(None, self.name)
 			# if we rename parent dir into a new location (simulate directory-base log rotation)
 			tmpsub1 = os.path.join(tmp, "1")
 			tmpsub2 = os.path.join(tmp, "2")
